/* * Copyright (c) 2024 Carl Klemm <carl@uvos.xyz>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation and/or
 * other materials provided with the distribution.
 *  * Neither the name of %ORGANIZATION% nor the names of its contributors may be
 * used to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#define _POSIX_C_SOURCE 199309L

#include <stdint.h>
#include "fandevice.h"
#include "usbshm.h"
#include <stdlib.h>
#include <time.h>
#include <stdio.h>

static void usleep(uint64_t microseconds)
{
	struct timespec ts;
	ts.tv_sec = microseconds / 1000000;
	ts.tv_nsec = (microseconds % 1000000) * 1000;
	nanosleep(&ts, NULL);
}

int fandevice_connect(struct fandevice* device, uint16_t serial)
{
	int ret;
	device->priv = malloc(sizeof(*device->priv));
	if(!device->priv)
		return -1;
	
	ret = usbshm_init(device->priv, NULL, device);
	if(ret)
		return -2;

	unsigned char serialStr[5];
	snprintf((char*)serialStr, sizeof(serialStr), "%04hu", serial);

	ret = usbshm_open(device->priv, 0xfe17, 0x28dc , serial == 0 ? NULL : serialStr);
	if(ret)
		return -3;
	return 0;
}

int fandevice_heartbeat(struct fandevice* device)
{
	int ret;
	while((ret = usbshm_writeControlTransfer(device->priv, COMMAND_HEARTBEAT, NULL, 0, 0, 0)) == USBSHM_ERROR_AGAIN)
		usleep(100000);
	return ret;
}

int fandevice_set_channel(struct fandevice* device, channel_t channel, float speed)
{
	if(channel == FAN_NONE || speed < 0 || speed > 1)
		return -1;

	uint8_t value = speed*255;

	int ret;
	while((ret = usbshm_writeControlTransfer(device->priv, COMMAND_SET_FAN, NULL, 0, value, (int)channel)) == USBSHM_ERROR_AGAIN)
		usleep(100000);
	return ret;
}

void fandevice_disconnect(struct fandevice* device)
{
	usbshm_distroy(device->priv);
	free(device->priv);
	device->priv = NULL;
}
